package com.recovery.common.base.config.feign;


import cn.dev33.satoken.same.SaSameUtil;
import cn.dev33.satoken.stp.StpUtil;
import com.recovery.common.base.util.HspHostUtil;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

public class CommonRequestInterceptor implements RequestInterceptor{

    @Override
    public void apply(RequestTemplate requestTemplate) {
        // SaSameToken 主要用于实现网关统一请求。即请求不能绕过网关去访问某个具体的服务
        requestTemplate.header(SaSameUtil.SAME_TOKEN, SaSameUtil.getToken());

        // 如果希望被调用方有会话状态，此处就还需要将 satoken 添加到请求头中
        requestTemplate.header(StpUtil.getTokenName(), StpUtil.getTokenValue());
        HttpServletRequest request = getHttpServletRequest();
        if(request != null && request.getHeader("hospitalHost") != null){
            String hospitalHost = request.getHeader("hospitalHost");

            String hospiatlProt = request.getHeader("hospiatlProt");
            String websiteDomain = request.getHeader("websiteDomain");
            String innerInternet = request.getHeader("innerInternet");
            String token = request.getHeader("userTokenHead");
            requestTemplate.header("hospitalHost", hospitalHost);
            requestTemplate.header("innerInternet", innerInternet);
            requestTemplate.header("websiteDomain",websiteDomain);
            requestTemplate.header("hospiatlProt",hospiatlProt);
            requestTemplate.header("userTokenHead",token);
            requestTemplate.header("feign","true");
        } else {
            //取当前线程变量中的域名进行传递
            requestTemplate.header("hospitalHost", HspHostUtil.getHspHost());
            requestTemplate.header("feign","true");
        }
    }

    private HttpServletRequest getHttpServletRequest() {
        try {
            ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            if(attributes != null){
                return attributes.getRequest();
            }else{
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

}
