package com.recovery.common.base.util;



import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.json.JSONObject;
import com.dtp.common.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class UtilTools {

	//protected final static Log log = LogFactory.getLog(UtilTools.class);

	private final static Pattern DECIMAL_NUMBER_PATTERN = Pattern.compile("[+-]?[0-9]+\\.([0-9]+)?");
	private final static Pattern DECIMAL_NUMBER_PATTERN_P1 = Pattern.compile("^[+]?(([1-9]\\d*[.]?)|(0.))(\\d{0,1})?$");
	private final static Pattern PATTERN_INTEGER = Pattern.compile("^[1-9]\\d*|0$");
	private final static Pattern PATTERN_PHONE = Pattern.compile("^((13[0-9])|(14[5,7,9])|(15([0-3]|[5-9]))|(166)|(17[0,1,3,5,6,7,8])|(18[0-9])|(19[0-9]))\\d{8}$");
	/**
	 * 验证数字
	 * 浮点数保留一位小数
	 * @param numberStr
	 * @return boolean
	 */
	public static boolean validateNumber(String numberStr){
		//String DECIMALREG = "^[+]?(([1-9]\\d*[.]?)|(0.))(\\d{0,1})?$";
		if(StringUtils.isEmpty(numberStr)) {
            return false;
        }
		//Pattern pattern = Pattern.compile(DECIMALREG);
		Matcher matcher = DECIMAL_NUMBER_PATTERN_P1.matcher(numberStr.trim());
		return matcher.matches();
	}

	/**
	 * 验证数字
	 * 正整数包括0
	 * @param numberStr
	 * @return boolean
	 */
	public static boolean validatePositiveInteger1(String numberStr){
		//String POSITIVEINTEGER1 = "^[1-9]\\d*|0$";
		if(StringUtils.isEmpty(numberStr)) {
            return false;
        }
		//Pattern pattern = Pattern.compile(POSITIVEINTEGER1);
		Matcher matcher = PATTERN_INTEGER.matcher(numberStr.trim());
		return matcher.matches();
	}

	//判断空
	public static boolean isEmpty(String str){

		if(null == str || str.trim().length() <= 0) {
            return true;
        }
		return false;
	}

	public static boolean isNotEmpty(String str){
		if(null == str || str.trim().length() <= 0) {
            return false;
        }
		return true;
	}

	public static boolean isNotEmpty(Object objStr){
		if(null == objStr || objStr.toString().trim().length() <= 0) {
            return false;
        }
		return true;
	}

	public static boolean isNotEmpty(Object... objStr){
		for(Object object : objStr){
			if(object == null){
				return false;
			}
		}
		return true;
	}


	public static boolean isEmpty(Double value){
		if(value == null){
			return true;
		}
		return false;
	}

	public static boolean isEmpty(Float str){
		if(null == str){
			return true;
		}
		return false;
	}

	public static boolean isEmpty(Long value){
		if(value == null){
			return true;
		}
		return false;
	}


	//判断date空
	public static boolean isEmpty(Date d){

		if(d == null){
			return true;
		}
		return false;
	}

	/**
	 * 判断Integer
	 * @param value
	 * @return
	 */
	public static boolean isEmpty(Integer value){
		if(value == null){
			return true;
		}
		return false;
	}

	public static boolean isEmpty(Object objStr){
		if(null == objStr || objStr.toString().trim().length() <= 0) {
            return true;
        }
		return false;
	}

	public static boolean isEmpty(List value){
		if(null == value || value.size() <= 0) {
            return true;
        }
		return false;
	}

	public static boolean isNotEmpty(List value){
		return !isEmpty(value);
	}

	public static Boolean isEmpty(BigDecimal value){
		if(null == value){
			return true;
		}
		return false;
	}

	//判断是否为数据
	public static boolean isNumeric(String str) {
		if(UtilTools.isEmpty(str)) {
            return false;
        }
		for (int i = str.length(); --i >= 0;) {
			if (!Character.isDigit(str.charAt(i))) {
				return false;
			}
		}
		return true;
	}


	//  GENERAL_PUNCTUATION 判断中文的“号

    //  CJK_SYMBOLS_AND_PUNCTUATION 判断中文的。号

    //  HALFWIDTH_AND_FULLWIDTH_FORMS 判断中文的，号

    public static boolean isChinese(char c) {

        Character.UnicodeBlock ub = Character.UnicodeBlock.of(c);

        if (ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS

                || ub == Character.UnicodeBlock.CJK_COMPATIBILITY_IDEOGRAPHS

                || ub == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS_EXTENSION_A

                || ub == Character.UnicodeBlock.GENERAL_PUNCTUATION

                || ub == Character.UnicodeBlock.CJK_SYMBOLS_AND_PUNCTUATION

                || ub == Character.UnicodeBlock.HALFWIDTH_AND_FULLWIDTH_FORMS) {

            return true;

        }

        return false;

    }


	// 判断是否为中文
	public static boolean isChinese(String strName) {
		boolean flag = false;
		char[] ch = strName.toCharArray();
		for (int i = 0; i < ch.length; i++) {
			char c = ch[i];
			if (isChinese(c) == true) {
				flag = true;
				return flag;
			} else {
				continue;
			}
		}
		return flag;
	}

    public static Long parseLong(String value){
    	if(value == null){
    		return null;
    	}
    	try{
    		return Long.parseLong(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static  Short parseShort(String value){
    	if(value == null){
    		return null;
    	}
    	try{
    		return Short.parseShort(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static Integer parseInteger(String value){
    	if(value == null){
    		return null;
    	}
    	try{
    		return Integer.parseInt(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static Double parseDouble(String value){
    	if(value == null){
    		return null;
    	}
    	try{
    		return Double.parseDouble(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static Float parseFloat(String value){
    	if(value == null){
    		return null;
    	}
    	try{
    		return Float.parseFloat(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static Date parseDate(String value,String pattern){
    	if(isEmpty(pattern)){
    		pattern = "yyyy-MM-dd";
    	}

    	if(value == null){
    		return null;
    	}

    	try{
    		SimpleDateFormat format = new SimpleDateFormat(pattern);
    		return format.parse(value);
    	}catch(Exception e){
    		return null;
    	}
    }


	public static Date parseDate2(String value,String pattern) throws ParseException {
		if(isEmpty(pattern)){
			pattern = "yyyy-MM-dd";
		}

		if(value == null){
			return null;
		}

		SimpleDateFormat format = new SimpleDateFormat(pattern);
		return format.parse(value);
	}

	public static Date parseDate(String value){
		Date date =  parseDate(value,"yyyy-MM-dd");
		if(null == date){
			date = parseDate(value,"yyyy/MM/dd");
		}
		return date;
    }


	public static boolean isValidDate(String str,String formatStr) {
		   boolean convertSuccess=true;
		   // 指定日期格式为四位年/两位月份/两位日期，注意yyyy/MM/dd区分大小写；
		   SimpleDateFormat format = new SimpleDateFormat(formatStr);
		   try{
			   format.setLenient(false);
			   format.parse(str);
		   } catch (Exception e){
		   	  return false;
		   }
		   return convertSuccess;
	}

    public static String formatDate(Date value,String pattern){
    	if(isEmpty(pattern)){
    		pattern = "yyyy-MM-dd";
    	}

    	if(value == null){
    		return null;
    	}

    	try{
    		SimpleDateFormat format = new SimpleDateFormat(pattern);
    		return format.format(value);
    	}catch(Exception e){
    		return null;
    	}
    }

    public static String formatDate(Date value){
    	return formatDate(value,"yyyy-MM-dd");
    }

	public static boolean isNumber(String arg){
		final String numbers = "1234567890.";
		if(arg == null || "".equals(arg)){
			return false;
		}
		char[] cs = arg.toCharArray();
		for(char c : cs ){
			int i = (int)c;
			if(numbers.indexOf(i) == -1){
				return false;
			}
		}
		return true;
	}

	public static boolean isPhone(String phone) {
		//String regex = "^((13[0-9])|(14[5,7,9])|(15([0-3]|[5-9]))|(166)|(17[0,1,3,5,6,7,8])|(18[0-9])|(19[0-9]))\\d{8}$";
		if (phone.length() != 11) {
			return false;
		} else {
			//Pattern p = Pattern.compile(regex);
			Matcher m = PATTERN_PHONE.matcher(phone);
			boolean isMatch = m.matches();
			return isMatch;
		}
	}





	public static String currValueConvertStr(String str){
		return UtilTools.isEmpty(str)?null:str.trim();
	}

	public  static boolean  isNegativeNumber(String arg){
		final String numbers = "-1234567890.";
		if(arg == null || "".equals(arg)){
			return false;
		}
		char[] cs = arg.toCharArray();
		for(char c : cs ){
			int i = (int)c;
			if(numbers.indexOf(i) == -1){
				return false;
			}
		}
		return true;
	}
	public static boolean isInt(String arg){
		final String numbers = "1234567890";
		if(arg == null || "".equals(arg)){
			return false;
		}
		char[] cs = arg.toCharArray();
		for(char c : cs ){
			int i = (int)c;
			if(numbers.indexOf(i) == -1){
				return false;
			}
		}
		return true;
	}

	public static String objectToString(Object obj){
		if(obj == null){
			return "";
		}
		return obj.toString();
	}

	/**
	 *
	 *@描述: 格式化数字
	 *@param num
	 *@param format
	 *@return
	 */
	public static String formatNum(Object num, String format){
		if(null==num){
			return "";
		}else if(null==format || format.equals("")){
			return num.toString();
		}
		DecimalFormat df  = new DecimalFormat(format);
		String text = df.format(num);
		return text;
	}

	public static List<String> stringToList(String str){
		if(!UtilTools.isEmpty(str)){
			String [] strs = str.split(",");
			if(null != strs && 0 != strs.length){
				List<String> returnVals = new ArrayList<String>();
				for(int i=0; i<strs.length; i++){
					if(isNotEmpty(strs[i])){
						returnVals.add(strs[i]);
					}
				}
				return returnVals;
			}
		}
		return new ArrayList<String>();
	}

	public static boolean isEmptyList(List list){
		if(null != list && list.size()>0){
			return false;
		}
		return true;
	}
	public static Long getJsonLong(JSONObject jb, String key){
		try {
			return jb.getLong(key);
		}catch (Exception e){
			return null;
		}
	}



	public static int getJsonInt(JSONObject jb,String key){
		try {
			return jb.getInt(key);
		}catch (Exception e){
			return 0;
		}
	}


	/**
	 * 提供（相对）精确的除法运算。当发生除不尽的情况时，由scale参数指
	 * 定精度，以后的数字四舍五入。
	 * @param v1 被除数
	 * @param v2 除数
	 * @param scale 表示表示需要精确到小数点以后几位。
	 * @return 两个参数的商
	 */
	public static double div(double v1,double v2,int scale){
		if(scale<0){
			throw new IllegalArgumentException(
					"The scale must be a positive integer or zero");
		}
		BigDecimal b1 = new BigDecimal(Double.toString(v1));
		BigDecimal b2 = new BigDecimal(Double.toString(v2));
		return b2.divide(b1,scale,BigDecimal.ROUND_HALF_UP).doubleValue();
	}

	public static String delHtmlTag(String str){
		String newstr = "";
		newstr = str.replaceAll("<[.[^>]]*>","");
		newstr = newstr.replaceAll(" ", "");
		return newstr;
	}



	public static String toUserName(String trueName){
		String userName = "";
		if(!isEmpty(trueName)){
			if(trueName.length() > 2){
				String name = trueName.substring(0,2);
				String[] arr={"欧阳","太史","上官","端木","司马","东方","独孤","南宫","万俟","闻人","夏侯","诸葛","尉迟","公羊","赫连","澹台","皇甫","宗政","濮阳","公冶","太叔","申屠","公孙","慕容","仲孙","钟离","长孙","宇文","司徒","鲜于","司空","闾丘","子车","亓官","司寇","巫马","公西","颛孙","壤驷","公良","漆雕","乐正","宰父","谷梁","拓跋","夹谷","轩辕","令狐","段干","百里","呼延","东郭","南门","羊舌","微生","公户","公玉","公仪","梁丘","公仲","公上","公门","公山","公坚","左丘","公伯","西门","公祖","第五","公乘","贯丘","公皙","南荣","东里","东宫","仲长","子书","子桑","即墨","达奚","褚师","吴铭"};
				Boolean flag = false;
				for (int i = 0; i < arr.length; i++) {
					if(name.equals(arr[i])){
						flag = true;
						userName = trueName.substring(2);
						break;
					}
				}
				if(!flag){
					userName = trueName.substring(1);
				}
			}else{
				if(trueName.length() > 1){
					userName = trueName.substring(1);
				}else{
					userName = trueName;
				}
			}
		}
		return userName;
	}

	public static String getObjectValue(Object obj){
		return obj==null?"":obj.toString();
	}

    /**
     * 将Object对象里面的属性和值转化成Map对象
     * @param obj
     * @return
     * @throws IllegalAccessException
     */
    public static Map<String, Object> objectToMap(Object obj)  {
        try{
            Map<String, Object> map = new HashMap<String,Object>();
            Class<?> clazz = obj.getClass();
            for (Field field : clazz.getDeclaredFields()) {
                field.setAccessible(true);
                String fieldName = field.getName();
                //Object value = StringUtils.isEmpty(field.get(obj));
                Object value = field.get(obj);
                map.put(fieldName, value);
            }
            return map;
        } catch (Exception e){
            log.error("将Object对象里面的属性和值转化成Map对象,{}", e.getMessage(), e);
            return null;
        }
    }

	/**
	 * 计算时间差
	 * @param startTime
	 * @param endTime
	 * @return
	 */
    public static int getDistanceTime(Date startTime, Date endTime){

		int days = 0;
		long time1 = startTime.getTime();
		long time2 = endTime.getTime();

		long diff;
		if (time1 < time2) {
			diff = time2 - time1;
		} else {
			return 0;
		}
		days = (int) (diff / (24 * 60 * 60 * 1000));
		return days;
	}




	/**
	 * 合并俩个对象的属性值[如果destination 属性不为空,将属性值合并到target]
	 * @param target
	 * @param destination
	 * @param <M>
	 * @throws Exception
	 */
	public static <M> void merge(M target, M destination) throws Exception {
		BeanInfo beanInfo = Introspector.getBeanInfo(target.getClass());

		// Iterate over all the attributes
		for (PropertyDescriptor descriptor : beanInfo.getPropertyDescriptors()) {
			// Only copy writable attributes
			if (descriptor.getWriteMethod() != null) {
				Object defaultValue = descriptor.getReadMethod().invoke(destination);
				// Only copy values values where the destination values is null
				if (defaultValue != null) {
					descriptor.getWriteMethod().invoke(target, defaultValue);
				}
			}
		}
	}

	/**
	 * 判断是否有小数
	 * @param bd
	 * @return
	 */
	public static boolean isIntegerValue(BigDecimal bd) {
		return bd.signum() == 0 || bd.scale() <= 0 || bd.stripTrailingZeros().scale() <= 0;
	}

	public static List<Object> splitToList(String str){
		List<Object> result = null;
		if(!isEmpty(str)){
			String[] arr = str.split(",");
			if(null != arr || arr.length > 0){
				result = new ArrayList<>();
				for(String s : arr){
					if(!isEmpty(s)){
						result.add(s);
					}
				}
			}
		}

		return result;
	}

	public static void main(String[] args) throws Exception{

		/*UserViewDto viewDto = new UserViewDto();
		UserDto userDto = new UserDto();
		userDto.setUserKey(DesUtil.encrypt("8"));
		viewDto.setUserDto(userDto);

		UtilTools.converCipherKeyToId(viewDto);*/

		//System.out.println(UtilTools.parseDate("2020-02-28T23:15:23.853","yyyy-MM-dd'T'HH:mm:ss.SSS"));

//		BigDecimal bd = new BigDecimal("1.0");
//		System.out.println(UtilTools.isIntegerValue(bd));

//		String a = ",a,a,a,";
//		System.out.println(splitToList(a)+"-----length:"+splitToList(a).size());

//		System.out.println(isNumeric("1500"));
//		System.out.println(isPhone("19012391943"));
//		System.out.println(isPhone("19112391943"));
//		System.out.println(isPhone("19212391943"));
//		System.out.println(isPhone("19312391943"));
//		System.out.println(isPhone("19412391943"));
//		System.out.println(isPhone("19512391943"));
//		System.out.println(isPhone("19612391943"));
//		System.out.println(isPhone("19712391943"));
//		System.out.println(isPhone("19812391943"));
//		System.out.println(isPhone("19912391943"));


	}


    /**
     * 去除字符串中所包含的空格（包括:空格(全角，半角)、制表符、换页符等）
     * @param s
     * @return
     */
    public static String removeAllBlank(String s){
        String result = "";
        if(null!=s && !"".equals(s)){
            result = s.replaceAll("[　*| *| *|//s*]*", "");
        }
        return result;
    }





	public static boolean isDecimal(String decimal) {
		// 非空校验
		if (StringUtils.isEmpty(decimal)) {
            return false;
        }

		// 小数校验（可以是+1.2,1.2,-1.2）
		//Pattern pattern = Pattern.compile("[+-]?[0-9]+\\.([0-9]+)?");
		if (!DECIMAL_NUMBER_PATTERN.matcher(decimal).matches()) {
            return false;
        }

		return true;
	}

	/**
	 * 验证是否为子网IP
	 * @param addr
	 * @param addr1
	 * @return addr is subnet address and addr1 is ip address. Function will return true, if addr1 is within addr(subnet)
	 */
	public static boolean netMatch(String addr, String addr1){

		String[] parts = addr.split("/");
		String ip = parts[0];
		int prefix;

		if (parts.length < 2) {
			prefix = 0;
		} else {
			prefix = Integer.parseInt(parts[1]);
		}

		Inet4Address a =null;
		Inet4Address a1 =null;
		try {
			a = (Inet4Address) InetAddress.getByName(ip);
			a1 = (Inet4Address) InetAddress.getByName(addr1);
		} catch (UnknownHostException e){}

		byte[] b = a.getAddress();
		int ipInt = ((b[0] & 0xFF) << 24) |
				((b[1] & 0xFF) << 16) |
				((b[2] & 0xFF) << 8)  |
				((b[3] & 0xFF) << 0);

		byte[] b1 = a1.getAddress();
		int ipInt1 = ((b1[0] & 0xFF) << 24) |
				((b1[1] & 0xFF) << 16) |
				((b1[2] & 0xFF) << 8)  |
				((b1[3] & 0xFF) << 0);

		int mask = ~((1 << (32 - prefix)) - 1);

		if ((ipInt & mask) == (ipInt1 & mask)) {
			return true;
		}
		else {
			return false;
		}
	}

	/***
	 * 判断map是否存在
	 * @param value
	 * @return
	 */
	public static boolean isEmpty(Map value){
		if(null == value || value.size() <= 0) {
            return true;
        }
		return false;
	}

	public static boolean isTrue (Boolean parm){
		if (parm == null){
			return false;
		}else {
            return parm;
        }
	}

	public static boolean isEqual (Boolean parm1, Boolean parm2 ){
		if (parm1 == null){
			if (parm2 == null){
				return true;
			}else {
				return false;
			}
		}else {
			if (parm2 == null){
				return false;
			}else {
				return Objects.equals(parm1, parm2);
			}
		}
	}

	/**
	 * 计算指定的日期加指定天数后的日期
	 * @param startDate 开始日期
	 * @param days 指定天数
	 * @return
	 */
	public static Date calculationDate(Date startDate, int days) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(startDate);
		calendar.add(Calendar.DATE,days);
		return calendar.getTime();
	}

	/**
	 * 获取两个日期相隔天数
	 * @param startDate  开始时间
	 * @param endDate  结束时间
	 * @return
	 */
	public static int getDaysBetween(Date startDate, Date endDate) {
		Calendar startCalendar = Calendar.getInstance();
		startCalendar.setTime(startDate);
		Calendar endCalendar = Calendar.getInstance();
		endCalendar.setTime(endDate);
		if (startCalendar.after(endCalendar)) {
			Calendar swap = startCalendar;
			startCalendar = endCalendar;
			endCalendar = swap;
		}
		int days = endCalendar.get(Calendar.DAY_OF_YEAR) - startCalendar.get(Calendar.DAY_OF_YEAR);
		int y2 = endCalendar.get(Calendar.YEAR);
		if (startCalendar.get(Calendar.YEAR) != y2) {
			startCalendar = (Calendar) startCalendar.clone();
			do {
				days += startCalendar.getActualMaximum(Calendar.DAY_OF_YEAR);//得到当年的实际天数  
				startCalendar.add(Calendar.YEAR, 1);
			} while (startCalendar.get(Calendar.YEAR) != y2);
		}
		return days;
	}




    /**
       * 获取某段时间内的周一（二等等）的日期
       * @param startDate 开始日期
       * @param endDate 结束日期
       * @param weekDays 获取周几，1－6代表周一到周六。0代表周日
       * @return 指定周的一段时间内的天数 在日期间隔内获取星期几的天数
       */
    public static int getDayNumberOfWeekWithinDateInterval(Date startDate, Date endDate, int weekDays) {
		SimpleDateFormat sd = new SimpleDateFormat("yyyy-MM-dd");
		String dataBegin = sd.format(startDate);
		String dataEnd = sd.format(endDate);
		int day = 0;
		Calendar cal = Calendar.getInstance();
		String[] dateInterval = {dataBegin, dataEnd};
		Date[] dates = new Date[dateInterval.length];
		for (int i = 0; i < dateInterval.length; i++) {
			String[] ymd = dateInterval[i].split("[^\\d]+");
			cal.set(Integer.parseInt(ymd[0]), Integer.parseInt(ymd[1]) - 1, Integer.parseInt(ymd[2]));
			dates[i] = cal.getTime();
		}
		for (Date date = dates[0]; date.compareTo(dates[1]) <= 0; ) {
			cal.setTime(date);
			if (cal.get(Calendar.DAY_OF_WEEK) - 1 == weekDays) {
				day += 1;
			}
			cal.add(Calendar.DATE, 1);
			date = cal.getTime();
		}
		return day;
    }

	public static Long getUserId(){
		try{
			log.info("当期库code："+HspHostUtil.getHspHost());
			String id = StpUtil.getLoginId()+"";
			if (StringUtil.isNotEmpty(id)) {
				Long userId = Long.parseLong(id.replace(HspHostUtil.getHspHost()+":",""));
				return userId;
			}else {
				log.info("获取当前登陆人id为空 当期库code："+HspHostUtil.getHspHost());
				return null;
			}
		}catch (Exception e){
			log.error("获取当前登陆人id失败",e.getMessage(),e);
			log.info("获取当前登陆人id失败");
            return null;
		}
	}
}
