package com.recovery.gateway.task;

import com.alibaba.fastjson.JSONObject;
import com.recovery.common.base.config.redis.BaseRedisCache;
import com.recovery.common.base.po.base.DBInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
public class DataInitTask {
    @Resource
    private BaseRedisCache redisCache;
    @Value("${master.datasource.url}")
    private String masterUrl;
    @Value("${master.datasource.username}")
    private String user;
    @Value("${master.datasource.password}")
    private String password;
    @Value("${master.datasource.driverClassName}")
    private String driverClass;
    /**
     * 初始医院DB配置
     */
    public void initHospitalDB(){
        try {
            JdbcTemplate jdbcTemplate=new JdbcTemplate(generateDataSource());
            List<Map<String,Object>> mapList = jdbcTemplate.queryForList("SELECT id,db_name,db_ip,db_port,db_user,db_passwd,`module`,hospital_host,hospital_code FROM db_info");
            if(mapList.size() > 0){
                Map<String,List<DBInfo>> dbMap = new HashMap<>();
                for(Map<String,Object> map : mapList){
                    DBInfo dbInfo = new DBInfo();
                    dbInfo.setId(Integer.parseInt(map.get("id").toString()));
                    dbInfo.setDbName(map.get("db_name").toString());
                    dbInfo.setDbIp(map.get("db_ip").toString());
                    dbInfo.setDbPort(Integer.parseInt(map.get("db_port").toString()));
                    dbInfo.setDbUser(map.get("db_user").toString());
                    dbInfo.setDbPasswd(map.get("db_passwd").toString());
                    dbInfo.setModule(map.get("module").toString());

                    if(map.get("hospital_host") != null) {
                        dbInfo.setHospitalHost(map.get("hospital_host").toString());
                    }
                    dbInfo.setHospitalCode((String)map.get("hospital_code"));
                    String codeKey = "hoe:hospital:dbInfo:"+dbInfo.getHospitalCode()+"_"+dbInfo.getModule();
                    redisCache.put(codeKey, JSONObject.toJSONString(dbInfo));
                    /** 废弃host字段 迁移至新表 website_info
                     if(UtilTools.isNotEmpty(dbInfo.getModule()) && dbInfo.getModule().equals("common")){
                     String hostKey = "hoe:hospital:host:"+dbInfo.getHospitalHost();
                     redisCache.put(hostKey,dbInfo.getHospitalCode());
                     redisCache.expireDays(hostKey,365);
                     }
                     **/
                    if(dbMap.containsKey(dbInfo.getModule())){
                        dbMap.get(dbInfo.getModule()).add(dbInfo);
                    }else{
                        List<DBInfo> dbs = new ArrayList<>();
                        dbs.add(dbInfo);
                        dbMap.put(dbInfo.getModule(),dbs);
                    }
                }
                dbMap.forEach((k,v)->{
                    String redisKey = "hoe:hospital:dbs:"+k;
                    redisCache.put(redisKey,JSONObject.toJSONString(v));
                });
            }
        }catch (Exception e){
            log.error("ERROR:initHospitalDB",e);
        }
    }

    private DriverManagerDataSource generateDataSource()throws Exception{
        DriverManagerDataSource dataSource=new DriverManagerDataSource();
        dataSource.setDriverClassName(driverClass);
        dataSource.setUrl(masterUrl);
        dataSource.setUsername(user);
        dataSource.setPassword(password);
        return dataSource;
    }
}
