package com.recovery.order.aspect;


import com.recovery.common.base.ds.DataSourceContextHolder;
import com.recovery.common.base.util.HspHostUtil;
import com.recovery.common.base.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.net.URL;

/**
 * Web层日志切面
 *
 * @author RyanWang
 * @version 1.0.0
 * @date 16/5/17 上午10:42.
 */
@Aspect
@Order(0)
@Slf4j
@Component
public class WebLogAspect {
    @Resource
    private StringRedisTemplate stringRedisTemplate;
//
    @Pointcut("execution(public * com.recovery.order.controller..*.*(..)) || execution(public * com.recovery.order.rest.*.*(..))")
    public void webLog(){}

    @Before("webLog()")
    public void doBefore(JoinPoint joinPoint) throws Throwable {

        // 接收到请求，记录请求内容
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        //获取请求的域名
        HttpServletRequest request = attributes.getRequest();
        // 根据域名获取网站信息
        StringBuffer urlBuf = request.getRequestURL();
        URL url = new URL(urlBuf.toString());
        String path = url.getPath();
        log.info("获取请求地址:" + path);
        log.info("获取请求ip:" + getIPAddress(request));
        HttpServletResponse response = (HttpServletResponse)attributes.getResponse();
        String hospitalHost = request.getHeader("hospitalHost");
//        if(StringUtils.isEmpty(hospitalHost)){
//            return;
//        }

        //切割获取访问目标模块
        String[] split = path.split("/");
        String module = split[0];
        log.info("切换库："+RedisUtils.getDBInfoByHostAndModule(stringRedisTemplate,hospitalHost,"admin"));
        //根据域名和请求的模块名查询目标数据库
        HttpSession session = request.getSession();
        /**
         * 切换为动态数据源实例
         */
        HspHostUtil.switchDB(hospitalHost,"admin",stringRedisTemplate);

    }
    public String getIPAddress(HttpServletRequest request) {
        String ipAddress = request.getRemoteAddr();
        return ipAddress;
    }

    //执行完切面后，将线程共享中的数据源名称清空
    @After("webLog()")
    public void after(JoinPoint joinPoint){
        DataSourceContextHolder.clearDBType();
    }
}

